/*********************************************************************
 *
 *                  BMS_CAN Header
 *
 *********************************************************************
 * FileName:        BMS_CAN.h
 * Processor:       PIC18F45K80
 * Complier:        Microchip C18 v3.41
 * Company:         KIT - CN - IPE
 *
 * Author           Date         Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Reiling V.       23.04.2012   Release
 *********************************************************************/

//#ifndef BMS_CAN_H          // To avoid duplicate inclusion
#define BMS_CAN_H



/*********************************************************************
*
*                             Defines
*
*********************************************************************/
#define CAN_OK              1
#define CAN_NOK             0

// 50 Fehlversuche entspricht 5,5ms oder CAN_Tx_Voltage von 8 Slaves
#define CAN_Tx_TimeOut    50u



/*********************************************************************
 *
 * General purpose typedef's
 *
 ********************************************************************/
typedef union _CAN_MAILBOX
{
  struct
  {
    uint8_t   BOX_No;
    uint32_t  CAN_Id;
    uint8_t   DLC;
    uint32_t  ACCEPTANCE_FILTER;
    int8_t    Data[8];
  };
} CAN_MAILBOX;

typedef struct
{
    uint8_t LIMITVOLTAGE;
    uint8_t SLAVE_SPI;
    uint8_t SLAVE_TIMING;
    uint8_t SLAVE_HARDWARE;
    uint8_t LIMITBOARDTEMP;
    uint8_t LIMITCELLTEMP;
    uint8_t LIMITCELLVOLTAGE;
} ERROR_flags;

/*********************************************************************
 *
 * external Globals
 *
 ********************************************************************/
extern uint32_t     CAN_BAL;
extern uint8_t      SlaveMode;
extern uint8_t      gCounter;
extern uint8_t      Alive_Slave;
extern uint8_t      Alive_Master;
extern uint8_t      Check_Alive_Master;
extern uint8_t      Count_Error_Master;
extern uint8_t      MuxCounter;





/*********************************************************************
 *
 * Funktions Prototypen
 *
 ********************************************************************/
int8_t CAN_Init              ( CAN_CONFIG  *config );
int8_t CAN_Init_Mailbox      ( CAN_MAILBOX *mbox);
int8_t CAN_Write             ( CAN_MAILBOX *mbox, uint8_t *data );
int8_t CAN_Read              ( CAN_MAILBOX *mbox, uint8_t *data );

int8_t CAN_Write_Voltage     ( void );
int8_t CAN_Write_Temperature ( void );
int8_t CAN_Write_Status      ( void );
int8_t CAN_Read_Master       ( void );
int8_t CAN_Read_Balancing    ( void );
int8_t CAN_Read_System       ( void );
int8_t CAN_Write_System      ( void );

int8_t Load_Mailbox          ( uint8_t SlaveNo );
