/*********************************************************************
 *
 *                  BMS_Slave Header
 *
 *********************************************************************
 * FileName:        BMS_Slave.h
 * Processor:       PIC18F45K80
 * Compiler:        Microchip C18 v3.41
 * Company:         KIT - CN - IPE
 *
 * Author           Date         Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Reiling V.       30.03.2012   Release
 *********************************************************************/

//#ifndef BMS_H      // To avoid duplicate inclusion
#define BMS_H


/*********************************************************************
 *
 * Include Files
 *
 ********************************************************************/
#include "P18F45K80.h"
#include "delays.h"
#include "BMS_ECAN.h"
#include "BMS_MISRA.h"
#include "BMS_NTC28.h"
#include "BMS_LTC.h"
#include "BMS_CAN.h"
#include "BMS_EEPROM.h"

/*********************************************************************
*
*                             Defines
*
*********************************************************************/
#define EV__TimeSlot30ms       0x0001
#define EV__TimeSlot100ms      0x0002
#define EV__TimeSlotT          0x0004
#define EV__TimeSlot1000ms     0x0008
#define EV__LTC_V_Start        0x0010
#define EV__LTC_V_Wait         0x0020
#define EV__LTC_T_Start        0x0040
#define EV__LTC_T_Wait         0x0080

#define CAN_Tx_ErrorCountMax   24u
#define CAN_Rx_ErrorCountMax   240u

#define STANDBY_MODE            0x01
#define CALIBRATION_MODE        0x02
#define RUN_MODE                0x04
#define ERROR_MODE              0x08
#define RESET_MODE              0x10
#define INITIALISATION_MODE     0x20

#define EE_DEFAULT_d            0xFF        // Default value EEPROM in debuggmode
#define EE_DEFAULT_f            0x00        // Default value EEPROM in flashmode
#define EE_LOAD                 0xAA        // EEPROM is written
#define EE_START_a              0x00        // EEPROM startadress
#define EE_308_a                0           // EEPROM startadress for frame 0x308
#define EE_309_a                8           // EEPROM startadress for frame 0x309

#define RUN_SLAVE               0x01
#define STOP_SLAVE              0x02
#define UNLOCK_SYSTEM_CAN       0x04
#define DISABLE_SYSTEM_CAN      0x08
#define SPI_SYNC_ON             0x10
#define SPI_SYNC_OFF            0x20
#define SYSTEM_IDSEARCH         0x40
#define SYSTEM_SERNRPRG         0x80
#define SYSTEM_GETSERNR         0x0A

//Limit min./max. Supply voltage in ADC value
#define SUPPLY_MIN              2250    //-10%         //2372 -5%
#define SUPPLY_MAX              2750    //+10%         //2622 +5%

/*********************************************************************
*
*                             Data Types
*
*********************************************************************/
typedef union
{
uint8_t EE_ADRESS[18];
    struct
    {
    uint8_t FLASH;
    uint8_t LFT1;
    uint8_t LFT2;
    uint8_t LFT3;
    uint8_t LFT4;
    uint8_t LIEF;
    uint8_t BAU1;
    uint8_t BAU2;
    uint8_t EE_REV;
    uint8_t FAM1;
    uint8_t FAM2;
    uint8_t GERAET;
    uint8_t LAEN;
    uint8_t SLAVE_ID;
    uint8_t LFT_1;
    uint8_t LFT_2;
    uint8_t BAU_1;
    uint8_t FAM_1;
    }EE_BYTE;
} SERIAL_NR_t;

/*********************************************************************
*
*                        Function Prototypes
*
*********************************************************************/
void  interrupt_at_low_vector ( void );
void  my_isr                  ( void );
void  ini                     ( uint8_t SlaveStatus );
void  main                    ( void );

